-- Functions that don't belong to a specific category

TFM_GetOrMakeChest = function(parent, index)
-- Gets the chest at the given index in the given parent. If it's nil, the chest is initialized.
	if parent == nil or index == nil then
		return nil
	end
	local chest = parent:Get_Chest(index)
	if chest == nil then
		chest = parent:Make_Chest(index)
	end
	return chest
end

TFM_ObjectDC = function(object)
-- Gives the functionalities of the old ObjectDC by using string representations of objects as indexes in the duel chest
	object = object or Object()
	if object == nil then
		return nil
	end
	local mainDC = TFM_GetOrMakeChest( MTG():DuelDataChest(), "TFM_ObjectDC" )
	local index = tostring(object)
	local objectDC = mainDC:Get_Chest(index)
	if objectDC == nil then
		objectDC = mainDC:Make_Chest(index)
	end
	return objectDC
end

TFM_RemoveCardType = function(object, type)
-- Removes the specified type from object, re-applying the other ones when needed
	local characteristics = object and object:GetCurrentCharacteristics()
	if characteristics ~= nil then
		local old_types = {}
		for t=0,8 do
			old_types[t] = object:GetCardType():Test(t)
		end
		local card_type = characteristics:CardType_GetWritable()
		card_type:Clear()
		for t=0,8 do
			if old_types[t] and t ~= type then
				card_type:Add(t)
			end
		end
	end
end

TFM_MessageBox = function(message, player, card)
-- Displays a simple message box with an OK button to the chosen player (can also display a card)
	message = message or "nil"
	player = player or EffectController()
	if player ~= nil then
		player:BeginNewMultipleChoice()
		player:AddMultipleChoiceAnswer("CONTROL_MB_CONFIRM")
		player:AskMultipleChoiceQuestion(message, card)
	end
end

TFM_GetManager = function(index, set, object)
-- Returns the manager registered for the chosen index.
-- If set is true, it also sets the object itself as manager if none has been found.
	local mainDC = TFM_GetOrMakeChest( MTG():DuelDataChest(), "TFM_Managers" )
	if set == true and mainDC:Get_CardPtr(index) == nil then
		object = object or Object()
		mainDC:Set_CardPtr(index, object)
	end
	return mainDC:Get_CardPtr(index)
end

TFM_HasCategories = function(object, categories)
-- Returns true if object has ALL the given categories (represented by an array of strings).
	if object == nil then
		return false
	end
	categories = categories or {}
	for i, cat in ipairs(categories) do
		if object:HasCategory(cat) == false then
			return false
		end
	end
	return true
end