-- Functions that manipulate counters

TFM_AddXpCounters = function(player, amount)
-- Adds the specified amount of experience counters to the player.
	player = player or EffectController()
	amount = amount or 1
	if player ~= nil and amount > 0 then
		local chest = player:PlayerDataChest()
		local emblem = chest:Get_CardPtr("TFM_XpEmblem")
		if emblem == nil then
			MTG():MakeEmblem("EMBLEM_EXPERIENCE", 1, player)
			emblem = chest:Get_CardPtr("TFM_XpEmblem")
		end
		emblem:AddCounters( MTG():GetCountersType("Experience"), amount )
	end
end

TFM_RemoveXpCounters = function(player, amount)
-- Removes the specified amount of experience counters from the player.
	player = player or EffectController()
	amount = amount or 1
	if player ~= nil and amount > 0 then
		local emblem = player:PlayerDataChest():Get_CardPtr("TFM_XpEmblem")
		if emblem ~= nil then
			emblem:RemoveCounters( MTG():GetCountersType("Experience"), amount )
		end
	end
end

TFM_CountXpCounters = function(player)
-- Returns the amount of experience counters on the player.
	player = player or EffectController()
	if player ~= nil then
		local emblem = player:PlayerDataChest():Get_CardPtr("TFM_XpEmblem")
		if emblem ~= nil then
			return emblem:CountCounters( MTG():GetCountersType("Experience") )
		end
	end
	return 0
end

TFM_MissingPoisonEmblem = function()
-- To be used in a PLAYER_POISON_CHANGED trigger condition, returns true if the poison emblem still needs to be spawned.
	local player = TriggerPlayer()
	if player ~= nil then
		return player:PlayerDataChest():Get_CardPtr("TFM_PoisonEmblem") == nil
	end
	return false
end

TFM_SpawnPoisonEmblem = function()
-- To be used in a PLAYER_POISON_CHANGED trigger, spawns the poison emblem.
-- Remember to check if this is necessary by checking TFM_MissingPoisonEmblem() inside the trigger condition.
	local player = TriggerPlayer()
	if player ~= nil then
		MTG():MakeEmblem("EMBLEM_POISON", 1, player)
		local emblem = player:PlayerDataChest():Get_CardPtr("TFM_PoisonEmblem")
		local poison = MTG():GetCountersType("Poison")
		local current_count = emblem:CountCounters(poison)
		emblem:AddCounters( poison, GetAmount()-current_count )
	end
end

TFM_CountPoisonCounters = function(player)
-- Returns the amount of poison counters on the player.
	player = player or EffectController()
	if player ~= nil then
		local emblem = player:PlayerDataChest():Get_CardPtr("TFM_PoisonEmblem")
		if emblem ~= nil then
			return emblem:CountCounters( MTG():GetCountersType("Poison") )
		end
	end
	return 0
end

TFM_Proliferate = function(team)
-- Runs the proliferate mechanic on permanents, then on experience/energy/poison.
	local my_team = team or EffectController():GetTeam()
	-- Step 1: permanents
	local filter = ClearFilter()
	filter:Add(FE_IS_PERMANENT, true)
	local filter_count = filter:EvaluateObjects()
	if filter_count > 0 then
		for i=0,filter_count-1 do
			local permanent = filter:GetNthEvaluatedObject(i)
			if permanent:CountCounters() > 0 then
				local suitable_counter_type = 0
				if permanent:GetController():GetTeam() == my_team then
					suitable_counter_type = permanent:GetBestOrWorstCounterType(1)
				else
					suitable_counter_type = permanent:GetBestOrWorstCounterType(0)
				end
				if suitable_counter_type ~= 0 then
					permanent:AddCounters(suitable_counter_type, 1)
				end
			end
		end
	end
	-- Step 2: experience, energy (in case of both, experience is deemed more valuable) and poison
	local n_players = MTG():GetNumberOfPlayers()
	for i=0,n_players-1 do
		local player = MTG():GetNthPlayer(i)
		if player ~= nil then
			if player:GetTeam() == my_team then
				if TFM_CountXpCounters(player) > 0 then
					TFM_AddXpCounters(player)
				elseif player:GetEnergyTotal() > 0 then
					player:GainEnergyCounters(1)
				end
			elseif TFM_CountPoisonCounters(player) > 0 then
				player:AddPoisonCounters(1)
			end
		end
	end
end